/* GTK Napster-0.301
 * Copyright (C) 2000 Unified (xilliator@hotmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>

#include "common.h"
#include "dlsess.h"

gboolean must_be_queued(gtk_sess *sess)
{
	gtk_dl_sess *dlsess;
	gint i = 0;
	gint nrofdls=0;

	while((dlsess = gtk_clist_get_row_data(GTK_CLIST(sess->transfer_list), i)) != NULL)
	{
		if((dlsess->ACTIVE) || (dlsess->CONNECTING)) nrofdls++;
		i++;
	}
	
	if(nrofdls < sess->maxdls) return FALSE;
	return TRUE;
}

void check_queue(gtk_sess *sess)
{
	gtk_dl_sess *dlsess;
	gint row;
	gchar *user;
	gchar *command;
	
	#ifdef NAPSTER_DEBUG
	g_print("MBQ: %d\n", must_be_queued(sess));
	g_print("GLL: %d\n", g_list_length(sess->queue));
	#endif

	if(must_be_queued(sess)) return;
	if(g_list_length(sess->queue) <= 0) return;

	dlsess = (gtk_dl_sess *)g_list_nth_data(sess->queue, 0);
	sess->queue = g_list_remove(sess->queue, (gpointer)dlsess);
	
	row = gtk_clist_find_row_from_data(GTK_CLIST(sess->transfer_list), dlsess);
	
	gtk_clist_get_text(GTK_CLIST(sess->transfer_list), row, 3, &user);

	command = (gchar *)g_malloc(strlen(dlsess->filename) + strlen(user) + 5);
	sprintf(command, "%s \"%s\"", user, dlsess->filename);

	if(!send_string(sess, command, 203))
	{
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
	}
	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_WAIT);
	dlsess->CONNECTING = TRUE;
	g_free(command);
}

void cancel_download_sess(gtk_dl_sess *dlsess)
{
	dlsess->sess->queue = g_list_remove(dlsess->sess->queue, (gpointer)dlsess);
	if(dlsess->ACTIVE)
	{
		gdk_input_remove(dlsess->socketid);
		close(dlsess->socketfd);
		fclose(dlsess->stream);
		dlsess->ACTIVE = FALSE;
	}
	if(dlsess->CONNECTING) dlsess->CANCEL=TRUE;
	check_queue(dlsess->sess);
} 

void selection_unselected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	gtk_sess *sess = (gtk_sess *)user_data;
	sess->current_sel = g_list_remove(sess->current_sel, GINT_TO_POINTER(row));
}

void selection_changed(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
	gtk_sess *sess = (gtk_sess *)data;
	sess->current_sel = g_list_append(sess->current_sel, GINT_TO_POINTER(row));
}

void transfer_selection_changed(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
	gtk_sess *sess = (gtk_sess *)data;
	sess->transfer_sel = g_list_append(sess->transfer_sel, GINT_TO_POINTER(row));
}

void transfer_selection_unselected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	gtk_sess *sess = (gtk_sess *)user_data;
	sess->transfer_sel = g_list_remove(sess->transfer_sel, GINT_TO_POINTER(row));
}

void library_selection_changed(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
	gtk_sess *sess = (gtk_sess *)data;
	sess->lib_selection = g_list_append(sess->lib_selection, GINT_TO_POINTER(row));
}

void library_selection_unselected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	gtk_sess *sess = (gtk_sess *)user_data;
	sess->lib_selection = g_list_remove(sess->lib_selection, GINT_TO_POINTER(row));
}

void upload_selection_changed(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
	gtk_sess *sess = (gtk_sess *)data;
	sess->ul_sel = g_list_append(sess->ul_sel, GINT_TO_POINTER(row));
}

void upload_selection_unselected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	gtk_sess *sess = (gtk_sess *)user_data;
	sess->ul_sel = g_list_remove(sess->ul_sel, GINT_TO_POINTER(row));
}

gchar *make_file(gchar *in)
{
	gint i;
	gint length = strlen(in);

	for(i=(length-1); i>0; i--)
		if(in[i] == '\\') break;
	if(in[i] == '\\') return (&in[i+1]);
	else return in;
}

void get_file_ok(gtk_sess *sess, gchar *instr)
{
	gint row = 0;
	gchar *filename = NULL;
	gchar *tspeed;
	gchar size[10];
	gchar *msg;
	gchar *fsize;
	gint speed;
	guint8 spacing;
	GdkPixmap *pixm;
	GdkBitmap *bitm;
	gtk_dl_sess *dlsess;
	gchar *seplist[6];
	
	/* 0=Username 1=IP 2=Port 3=Filename 4=Md5 5=Speed */
	seperate_string(instr, (gchar **)seplist, 6);
	speed = atoi(seplist[5]);
	tspeed = get_speed(speed);	

	while(1)
	{
		dlsess = gtk_clist_get_row_data(GTK_CLIST(sess->transfer_list), row);
		if(dlsess == NULL)
		{
			g_warning("Couldn't find %s in list.", seplist[3]);
			check_queue(sess);
			return;
		}
		if(strcasecmp(dlsess->filename, seplist[3]) == 0) break;
		row++;
	}
	
	if(dlsess->ACTIVE)
	{
		dlsess->CONNECTING = FALSE;
		check_queue(sess);
		return;
	}

	if((!dlsess->CONNECTING) || dlsess->CANCEL)
	{
		if(dlsess->CLEAR)
		{
			g_free(dlsess->filename);
			g_free(dlsess);
		}
		check_queue(sess);
		return;
	}

	if(gtk_clist_get_pixtext(GTK_CLIST(sess->transfer_list), row, 0, &filename, (guint8 *)&spacing, &pixm, &bitm) == 0)
	{
		g_warning("Error finding filename...");
		gtk_clist_set_text(GTK_CLIST(sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		check_queue(sess);
		return;
	}
	
	sprintf((gchar *)dlsess->md5, "%s", seplist[4]);
	
	if(strcmp(seplist[2], "0") == 0)
	{
		msg = (gchar *)g_malloc(strlen(seplist[0]) + strlen(seplist[3]) + 4);
		sprintf(msg, "%s \"%s\"", seplist[0], seplist[3]);
		if(!send_string(sess, msg, 500))
		{
			g_free(msg);
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
			dlsess->CONNECTING = FALSE;
			check_queue(sess);
			return;
		}
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 4, tspeed);
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_FIREWALL);
		g_free(msg);
		return;
	}
	
	if(connect_to_fileserver(dlsess, seplist[1], seplist[2], seplist[3], row, filename))
	{
		sprintf(size, "%d", (gint)dlsess->size);
		fsize = make_text_size(dlsess->size);
		if(dlsess->sess->SHOW_BYTE)	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 1, fsize);
		else gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 1, size);
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 4, tspeed);
		dlsess->start_time = time(NULL);
		dlsess->ACTIVE = TRUE;
		g_free(fsize);
	}
	dlsess->CONNECTING = FALSE;
	if(dlsess->CLEAR)
	{
		g_free(dlsess->filename);
		g_free(dlsess);
	}
	check_queue(sess);
}

void file_to_read(gpointer data, gint source, GdkInputCondition condition)
{
	gtk_dl_sess *dlsess = (gtk_dl_sess *)data;
	gchar buffer[4096];
	gchar percent_done[15];
	gint length;
	gint row = 0;
	gint percent;
	gdouble dlrate;
	
	row = gtk_clist_find_row_from_data(GTK_CLIST(dlsess->sess->transfer_list), dlsess);
	
	if((length = recv(dlsess->socketfd, buffer, 4096, 0)) == -1)
	{
		if((dlsess->current) < dlsess->size)
		{
			perror("recv");
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
			cancel_download_sess(dlsess);
		}
		else
		{
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 2, "100%");
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_FINISH);
			cancel_download_sess(dlsess);
		}
		return;
	}

	if(length == 0)
	{
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 2, "100%");
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_FINISH);
		cancel_download_sess(dlsess);
		return;
	}
	
	fwrite(buffer, 1, length, dlsess->stream);
	dlsess->current += length;
	dlrate = difftime(time(NULL), dlsess->start_time);

	if(dlrate != 0) dlrate = ((gdouble)dlsess->current/1024)/dlrate;
	percent = (gint)((dlsess->current*100)/dlsess->size);
	sprintf(percent_done, "%d%% %.2f KB/s", percent, dlrate);
	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 2, percent_done);
}

gint connect_to_fileserver(gtk_dl_sess *dlsess, gchar *ip, gchar *port, gchar *file, gint row, gchar *filename)
{
	struct sockaddr_in dest_addr;
	unsigned short int sport;
	gint sendt = 0;
	gint length;
	gchar *msg = "GET";
	gchar buffer[20];
	gchar *command;
	gint status;
	time_t start_time;

	sport = atoi(port);
	
	dest_addr.sin_family = AF_INET;
	
	dest_addr.sin_port = htons(sport);
	dest_addr.sin_addr.s_addr = (unsigned long int)strtoul(ip, NULL, 10);

	#ifdef LOCAL_TEST
	dest_addr.sin_port = htons(6699);
	dest_addr.sin_addr.s_addr = inet_addr("192.168.0.3");
	#endif
	
	memset(dest_addr.sin_zero, 0, 8);
	
	#ifdef NAPSTER_DEBUG
	g_print("\nIP: %s PORT: %s\n", ip, port);
	#endif
	
	if((dlsess->socketfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
  	  	gtk_statusbar_push(GTK_STATUSBAR(dlsess->sess->statusbar), 1, STATUSBAR_ERROR_TRANSFER);
  	  	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		return FALSE;
	}

	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_CONNECTING);
	
	start_time = time(NULL);
	status = connect_with_select(dlsess->socketfd, &dest_addr, start_time, dlsess->sess->timeout, &dlsess->CANCEL);
	
	if(dlsess->CLEAR)
	{
		close(dlsess->socketfd);
		return FALSE;
	}
	
	row = gtk_clist_find_row_from_data(GTK_CLIST(dlsess->sess->transfer_list), dlsess);

	switch(status)
	{
		case RECV_SELECT_TIMOUT:
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_TIMEOUT);
			close(dlsess->socketfd);
			return FALSE;
		case RECV_SELECT_ERROR:
	  	  	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
			close(dlsess->socketfd);
			return FALSE;
		case RECV_SELECT_CANCEL:
	  	  	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_CANCEL);
			close(dlsess->socketfd);
			return FALSE;
	}
	
	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_GET);
   
	length = recv_with_select(dlsess->socketfd, (gchar *)buffer, 1, start_time, dlsess->sess->timeout, &dlsess->CANCEL);

	if(dlsess->CLEAR)
	{
		close(dlsess->socketfd);
		return FALSE;
	}
	
	row = gtk_clist_find_row_from_data(GTK_CLIST(dlsess->sess->transfer_list), dlsess);

   switch (length)
	{
		case RECV_SELECT_TIMOUT:
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_TIMEOUT);
			close(dlsess->socketfd);
			return FALSE;
		case RECV_SELECT_ERROR:
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
			close(dlsess->socketfd);
			return FALSE;
		case RECV_SELECT_CANCEL:
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_CANCEL);
			close(dlsess->socketfd);
			return FALSE;			
	}

	if(buffer[0] != '1')
	{
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		close(dlsess->socketfd);
		return FALSE;
	}

	if((send(dlsess->socketfd, msg, 3, 0)) == -1)
	{
		perror("send");
  	  	gtk_statusbar_push(GTK_STATUSBAR(dlsess->sess->statusbar), 1, STATUSBAR_ERROR_GET);
  	  	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		close(dlsess->socketfd);
		return FALSE;
	}
	
	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_OK);

	command = (gchar *)g_malloc(strlen((gchar *)dlsess->sess->username) + strlen(file) + 20);
	/* sprintf(command, "%s \"%s\" %.0f", (gchar *)dlsess->sess->username, file, (gdouble)dlsess->current); */
	sprintf(command, "%s \"%s\" 0", (gchar *)dlsess->sess->username, file);
	length = strlen(command);

	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_FILE);

	if((sendt += send(dlsess->socketfd, command, length, 0)) == -1)
	{
		perror("send");
		g_free(command);
  		gtk_statusbar_push(GTK_STATUSBAR(dlsess->sess->statusbar), 1, STATUSBAR_ERROR_FILENAME);
  		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		close(dlsess->socketfd);
		return FALSE;
	}
  
	g_free(command);

	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_SIZE);

	length = recv_with_select(dlsess->socketfd, (gchar *)buffer, 20, start_time, dlsess->sess->timeout, &dlsess->CANCEL);

	if(dlsess->CLEAR)
	{
		close(dlsess->socketfd);
		return FALSE;
	}
	
	row = gtk_clist_find_row_from_data(GTK_CLIST(dlsess->sess->transfer_list), dlsess);

   switch (length)
	{
		case RECV_SELECT_TIMOUT:
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_TIMEOUT);
			close(dlsess->socketfd);
			return FALSE;
		case RECV_SELECT_ERROR:
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
			close(dlsess->socketfd);
			return FALSE;
		case RECV_SELECT_CANCEL:
			gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_CANCEL);
			close(dlsess->socketfd);
			return FALSE;			
	}

	buffer[length] = '\0';

	if(strcmp(buffer, "INVALID REQUEST") == 0) 
	{
		gtk_statusbar_push(GTK_STATUSBAR(dlsess->sess->statusbar), 1, STATUSBAR_ERROR_INVALID_REQUEST);
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		close(dlsess->socketfd);
		return FALSE;
	}
	
	if(strcmp(buffer, "FILE NOT SHARED") == 0) 
	{
		gtk_statusbar_push(GTK_STATUSBAR(dlsess->sess->statusbar), 1, STATUSBAR_ERROR_NOT_SHARED);
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		close(dlsess->socketfd);
		return FALSE;
	}
	
	if(strcmp(buffer, "FILE NOT FOUND") == 0) 
	{
		gtk_statusbar_push(GTK_STATUSBAR(dlsess->sess->statusbar), 1, STATUSBAR_ERROR_NOT_FOUND);
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		close(dlsess->socketfd);
		return FALSE;
	}
	
	command = (gchar *)g_malloc(strlen(filename) + strlen(dlsess->sess->directory) + 2);
	
	strcpy(command, dlsess->sess->directory);
	if(dlsess->sess->directory[strlen(dlsess->sess->directory)-1] != '/')
		strcat(command, "/");
	strcat(command, filename);

	if((dlsess->stream = fopen(command, "wb")) == NULL)
	{
		gtk_statusbar_push(GTK_STATUSBAR(dlsess->sess->statusbar), 1, STATUSBAR_ERROR_FILE);
		g_print(command);
		perror("fopen");
		g_free(command);
		gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_ERROR);
		close(dlsess->socketfd);
		return FALSE;
	}
	
	g_free(command);
	/*fseek(dlsess->stream, dlsess->current, SEEK_SET);*/
	dlsess->size = atol(buffer);
	dlsess->socketid = gdk_input_add(dlsess->socketfd, GDK_INPUT_READ, GTK_SIGNAL_FUNC(file_to_read), dlsess);	

	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 5, TRANSFER_STATUS_DOWNLOAD);

	return TRUE;
}

void dl_incomming(gtk_sess *sess, gint socketfd, time_t start_time)
{
	gtk_dl_sess *dlsess;
	gchar buffer[500];
	gchar *tsize;
	gchar *seplist[3];
	gchar offset[10];
	gchar *msg;
	gchar *filename;
	guint8 spacing;
	GdkPixmap *pixm;
	GdkBitmap *bitm;
	gint row = 0;
	gint length;
	gboolean CANCEL = FALSE;

	/*	0=Username  1=File  2=Size 	*/
	
	switch (length = recv_with_select(socketfd, (gchar *)buffer, 500, start_time, sess->timeout, &CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			g_warning("Firewall request timed out.");
			close(socketfd);
			return;
		case RECV_SELECT_ERROR:
			g_warning("Firewall error: Some strange socket error.");
			close(socketfd);
			return;
	}
	
	
	seperate_string((gchar *)buffer, (gchar **)seplist, 3);

	while(1)
	{
		dlsess = gtk_clist_get_row_data(GTK_CLIST(sess->transfer_list), row);
		if(dlsess == NULL)
		{
			send(socketfd, "INVALID REQUEST", strlen("INVALID REQUEST"), 0);
			g_warning("Firewall error: Didn't find file in list");
			close(socketfd);
			dlsess->CONNECTING = FALSE;
			check_queue(sess);
			return;
		}
		if(strcasecmp(dlsess->filename, seplist[1]) == 0) break;
		row++;
	}
	
	if(gtk_clist_get_pixtext(GTK_CLIST(sess->transfer_list), row, 0, &filename, (guint8 *)&spacing, &pixm, &bitm) == 0)
	{
			send(socketfd, "INVALID REQUEST", strlen("INVALID REQUEST"), 0);
			g_warning("Firewall error: Can't find filename");
			close(socketfd);
			dlsess->CONNECTING = FALSE;
			check_queue(sess);
			return;
	}

	if(dlsess->CANCEL || (!dlsess->CONNECTING))
	{
		close(dlsess->socketfd);
		if(dlsess->CLEAR)
		{
			g_free(dlsess->filename);
			g_free(dlsess);
		}
		check_queue(sess);
		return;
	}

	if(dlsess->ACTIVE) 
	{
		send(socketfd, "INVALID REQUEST", strlen("INVALID REQUEST"), 0);
		g_warning("Firewall error: Allready active!");
		close(socketfd);
		dlsess->CONNECTING = FALSE;
		check_queue(sess);
		return;
	}

	dlsess->socketfd = socketfd;
	dlsess->size = atol(seplist[2]);
	
	tsize = make_text_size(dlsess->size);
	if(dlsess->sess->SHOW_BYTE)	gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 1, tsize);
	else gtk_clist_set_text(GTK_CLIST(dlsess->sess->transfer_list), row, 1, seplist[2]);
	g_free(tsize);

	/* sprintf((gchar *)offset, "%.0f", (gdouble)dlsess->current); */
	sprintf((gchar *)offset, "0");

	if((send(socketfd, offset, strlen(offset), 0)) == -1)
	{
		perror("send");
		close(socketfd);
		dlsess->CONNECTING = FALSE;
		check_queue(sess);
		return;
	}
	
	msg = (gchar *)g_malloc(strlen(filename) + strlen(sess->sharedir) + 2);
	
	strcpy(msg, sess->sharedir);
	if(sess->directory[strlen(sess->sharedir)-1] != '/')
		strcat(msg, "/");
	strcat(msg, filename);
	
	if((dlsess->stream = fopen(msg, "wb")) == NULL)
	{
		g_warning("Can't open file: %s", msg);
		perror("fopen");
		close(socketfd);
		g_free(msg);
		dlsess->CONNECTING = FALSE;
		check_queue(sess);
		return;
	}

	g_free(msg);

	/* fseek(dlsess->stream, dlsess->current, SEEK_SET); */

	dlsess->ACTIVE = TRUE;
	dlsess->socketid = gdk_input_add(dlsess->socketfd, GDK_INPUT_READ, GTK_SIGNAL_FUNC(file_to_read), dlsess);
	dlsess->CONNECTING = FALSE;
}

