/* GTK Napster-0.301
 * Copyright (C) 2000 Unified (xilliator@hotmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <string.h>
#include <stdlib.h>
#include <sys/time.h>

#include "common.h"
#include "ncommand.h"

gboolean seperate_string(gchar *instr, gchar **outlist, gint nr)
{
	gint i;
	gchar *temp;
	
	outlist[0] = instr;
	for(i=0; i<nr; i++)
	{
		if(outlist[i][0] == '\"')
		{
			outlist[i] = outlist[i]+1;
			temp = strchr(outlist[i], '\"');
			temp[0] = '\0';
			if(i<(nr-1))
				outlist[i+1] = temp+2;
		}
		else
		{
			if(i<(nr-1))
			{
				temp = strchr(outlist[i], ' ');
				if(temp != NULL)
				{
					temp[0] = '\0';
					outlist[i+1] = temp+1;
				}
				else return FALSE;
			}
		}
	}
	return TRUE;
}

void string_out(GtkWidget *widget, gchar *instr, gint red, gint green, gint blue)
{
	GdkColormap *cmap;
	GdkColor color;
	
	cmap = gdk_colormap_get_system();
	color.red = red;
	color.green = green;
	color.blue = blue;
	
	if (!gdk_color_alloc(cmap, &color))
		g_error(COLOR_ALLOC_ERROR);
		
	gtk_text_insert (GTK_TEXT (widget), NULL, &color, NULL, instr, -1);
	gtk_text_insert (GTK_TEXT (widget), NULL, &color, NULL, "\n", -1);
}

void red_print(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	
	chatbox = get_chat_box_nr(sess, PRIVATE_CHAT_WINDOW);
	
	if(chatbox != -1)
	{
		string_out(sess->chatwins[chatbox].ctext, instr, 0xffff, 0, 0);
	}
	else
	{
		g_warning(WARNING_WRITE_CHANNEL);
	}
}

void ping_back(gtk_sess *sess, gchar *instr, gboolean you)
{
	gint chatbox;
	gchar *msg;
	
	chatbox = get_chat_box_nr(sess, PRIVATE_CHAT_WINDOW);
	if(chatbox != -1)
	{
		if(!you)
		{
			msg = (gchar *)g_malloc(strlen(PING_REPLY_USER) + strlen(instr));
			sprintf((gchar *)msg, PING_REPLY_USER, instr);
		}
		else
		{
			msg = (gchar *)g_malloc(strlen(PING_REPLY_SERVER) + strlen(instr));
			sprintf((gchar *)msg, PING_REPLY_SERVER, instr);
		}
		string_out(sess->chatwins[chatbox].ctext, msg, 200, 200, 200);
		g_free(msg);
	}
	else
	{
		g_warning(WARNING_WRITE_CHANNEL);
	}
}

void unknown_command(gtk_sess *sess, gchar *instr, gint code)
{
	g_print(UNKNOWN_COMMAND, code, instr);
}

char *get_speed(gint speed)
{
	switch(speed)
	{
		case 0: return LINE_SPEED_UNKNOWN;
		case 1: return LINE_SPEED_14MODEM;
		case 2: return LINE_SPEED_28MODEM;
		case 3: return LINE_SPEED_33MODEM;
		case 4: return LINE_SPEED_56MODEM;
		case 5: return LINE_SPEED_ISDN56;
		case 6: return LINE_SPEED_ISDN128;
		case 7: return LINE_SPEED_CABLE;
		case 8: return LINE_SPEED_DSL;
		case 9: return LINE_SPEED_T1;
		case 10: return LINE_SPEED_T3;
	};
	return STRANGE_LINESPEED;
}

gint get_speed_nr(gchar *in)
{
	if(strcmp(in, LINE_SPEED_UNKNOWN) == 0) return 0;
	else if(strcmp(in, LINE_SPEED_14MODEM) == 0) return 1;
	else if(strcmp(in, LINE_SPEED_28MODEM) == 0) return 2;
	else if(strcmp(in, LINE_SPEED_33MODEM) == 0) return 3;
	else if(strcmp(in, LINE_SPEED_56MODEM) == 0) return 4;
	else if(strcmp(in, LINE_SPEED_ISDN56) == 0) return 5;
	else if(strcmp(in, LINE_SPEED_ISDN128) == 0) return 6;
	else if(strcmp(in, LINE_SPEED_CABLE) == 0) return 7;
	else if(strcmp(in, LINE_SPEED_DSL) == 0) return 8;
	else if(strcmp(in, LINE_SPEED_T1) == 0) return 9;
	else if(strcmp(in, LINE_SPEED_T3) == 0) return 10;
	else return 0;
}

void command_connect(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	gint row;
	gint speed;
	gchar *tspeed;
	gchar *listitem[3];
	gchar *msg;
	gchar *seplist[4];
		
	/*	0=Channel  1=User  2=Songs  3=Speed 	*/

	msg = (gchar *)g_malloc(strlen(USER_JOIN_MSG) + strlen(instr));
	seperate_string(instr, (gchar **)seplist, 4);

	speed = atoi(seplist[3]);
	tspeed = get_speed(speed);
	
	listitem[0] = seplist[1];
	listitem[1] = seplist[2];
	listitem[2] = tspeed;

	chatbox = get_chat_box_nr(sess, seplist[0]);
	if(chatbox == -1)
	{
		g_free(msg);
		g_warning(WARNING_WRITE_CHANNEL);
		return;
	}

	sprintf((gchar *)msg, USER_JOIN_MSG, seplist[1], seplist[2], tspeed);
	string_out(sess->chatwins[chatbox].ctext, (gchar *)msg, 0, 0, 0xffff);
	row = gtk_clist_append(GTK_CLIST(sess->chatwins[chatbox].clist), listitem);
	if(speed < 4) gtk_clist_set_pixtext(GTK_CLIST(sess->chatwins[chatbox].clist), row, 0, listitem[0], 2, sess->pixmaps.pm_red, sess->pixmaps.bm_red );
	else if(speed < 7) gtk_clist_set_pixtext(GTK_CLIST(sess->chatwins[chatbox].clist), row, 0, listitem[0], 2, sess->pixmaps.pm_gold, sess->pixmaps.bm_gold );
	else gtk_clist_set_pixtext(GTK_CLIST(sess->chatwins[chatbox].clist), row, 0, listitem[0], 2, sess->pixmaps.pm_green, sess->pixmaps.bm_green );
	g_free(msg);
}

void destroy_list(gpointer data)
{
	gchar *temp = (gchar *)data;
	g_free(temp);
}

void command_search_results(gtk_sess *sess, gchar *instr)
{
	gchar *seplist[9];
	gchar *inlist[7];
	gchar *tspeed;
	gchar *time;
	gchar *size;
	gchar *data;
	gint row;
	gint speed;
	
	seperate_string(instr, (gchar **)seplist, 9);

	data = (gchar *)g_malloc(strlen(seplist[0]) + 1);
	strcpy(data, seplist[0]);

	speed = atoi(seplist[8]);
	tspeed = get_speed(speed);

	time = make_text_time((glong)atoi(seplist[5]), sess->SHOW_DHMS);
	size = make_text_size((glong)atoi(seplist[2]));

	inlist[0] = make_file(seplist[0]);
	if(sess->SHOW_BYTE) inlist[1] = size;
	else inlist[1] = seplist[2];
	inlist[2] = seplist[3];
	inlist[3] = seplist[4];
	inlist[4] = time;
	inlist[5] = seplist[6];
	inlist[6] = tspeed;
	
	row = gtk_clist_append(GTK_CLIST(sess->clist), inlist);
	if(speed < 4) gtk_clist_set_pixtext(GTK_CLIST(sess->clist), row, 0, inlist[0], 2, sess->pixmaps.pm_red, sess->pixmaps.bm_red );
	else if(speed < 7) gtk_clist_set_pixtext(GTK_CLIST(sess->clist), row, 0, inlist[0], 2, sess->pixmaps.pm_gold, sess->pixmaps.bm_gold );
	else gtk_clist_set_pixtext(GTK_CLIST(sess->clist), row, 0, inlist[0], 2, sess->pixmaps.pm_green, sess->pixmaps.bm_green );
	gtk_clist_set_row_data_full(GTK_CLIST(sess->clist), row, data, destroy_list);
	
	g_free(size);
	g_free(time);
}

void command_search_notfound(gtk_sess *sess)
{
	gtk_clist_set_text(GTK_CLIST(sess->clist), 0, 0, CLIST_SEARCH_FINISHED);
}

void command_resume_notfound(gtk_sess *sess)
{
	sess->resume.RECEIVING = FALSE;
	if(sess->resume.OPEN) gtk_label_set_text(GTK_LABEL(sess->resume.status), RESUME_FINISHED);
}

void command_leave(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	gchar *name;
	gint i=0;
	gchar *msg;
	gchar *tspeed;
	GdkPixmap *pixm;
	GdkBitmap *bitm;
	gint spacing;
	gint speed;
	gchar *seplist[4]; 
	
	/*	0=Channel  1=User  2=Songs  3=Speed 	*/
	
	msg = (gchar *)g_malloc(strlen(USER_LEAVE_MSG) + strlen(instr));
	seperate_string(instr, (gchar **)seplist, 4);
	
	chatbox = get_chat_box_nr(sess, seplist[0]);
	speed = atoi(seplist[3]);
	tspeed = get_speed(speed);
	
	if(chatbox == -1)
	{
		g_warning(WARNING_WRITE_CHANNEL);
		g_free(msg);
		return;
	}
	
	sprintf(msg, USER_LEAVE_MSG, seplist[1], seplist[2], tspeed);
	string_out(sess->chatwins[chatbox].ctext, msg, 0xffff, 0, 0);

	do
	{
		gtk_clist_get_pixtext(GTK_CLIST(sess->chatwins[chatbox].clist), i, 0, &name, (guint8 *)&spacing, &pixm, &bitm);
		if(strcmp(name, seplist[1]) == 0)
		{
			gtk_clist_remove(GTK_CLIST(sess->chatwins[chatbox].clist), i);
			break;
		}
		i++;
	} while(name != NULL);
	g_free(msg);
}

void command_whois_ans(gtk_sess *sess, gchar *instr)
{
	gchar *tspeed;
	gchar *msg;
	gint speed;
	gchar *time;
	gchar *seplist[10];
	gint size=0;
		
	/* 0=User 1=Level 2=Time 3=Channel 4=Ability 5=Shared 6=DL 7=UL 8=Speed 9=Version */
	
	size = strlen(instr);
	
	seperate_string(instr, (gchar **)seplist, 10);
	time = make_text_time(atoi(seplist[2]), TRUE);
	msg = (gchar *)g_malloc(size + strlen(USER_WHOIS_MSG) + strlen(time));

	speed = atoi(seplist[8]);
	tspeed = get_speed(speed);
	sprintf(msg, USER_WHOIS_MSG, seplist[0], seplist[1], time, seplist[3], seplist[4], seplist[5], seplist[6], seplist[7], tspeed, seplist[9]);
	gtk_message_whois(msg, MSGBOX_WHOIS_TITLE);
	g_free(msg);
	g_free(time);	
}

void command_whois_offline_ans(gtk_sess *sess, gchar *instr)
{
	gchar *msg;
	gchar *time;
	time_t leavetime;
	gchar *seplist[3];
	gint size=0;
		
	/* 0=User 1=Level 2=Time */
	
	size = strlen(instr);
	
	seperate_string(instr, (gchar **)seplist, 10);
	leavetime = (time_t)atoi(seplist[2]);
	if(leavetime == 0) time = USER_WHOIS_OFF_UNKNOWN;
	else time = ctime(&leavetime);
	msg = (gchar *)g_malloc(size + strlen(USER_WHOIS_OFF_MSG) + strlen(time));

	sprintf(msg, USER_WHOIS_OFF_MSG, seplist[0], seplist[1], time);
	gtk_message_whois(msg, MSGBOX_WHOIS_OFF_TITLE);
	g_free(msg);
}

void command_join_list(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	gint row;
	gint speed;
	gchar *tspeed;
	gchar *listitem[3];
	gchar *seplist[4];
	
	/*	0=Channel  1=User  2=Songs  3=Speed 	*/

	seperate_string(instr, (gchar **)seplist, 4);

	speed = atoi(seplist[3]);
	tspeed = get_speed(speed);
	
	listitem[0] = seplist[1];
	listitem[1] = seplist[2];
	listitem[2] = tspeed;

	chatbox = get_chat_box_nr(sess, seplist[0]);
	if(chatbox == -1)
	{
		g_warning(WARNING_GET_CHANNEL, seplist[0]);
		return;
	}
	
	row = gtk_clist_append(GTK_CLIST(sess->chatwins[chatbox].clist), listitem);
	if(speed < 4) gtk_clist_set_pixtext(GTK_CLIST(sess->chatwins[chatbox].clist), row, 0, listitem[0], 2, sess->pixmaps.pm_red, sess->pixmaps.bm_red );
	else if(speed < 7) gtk_clist_set_pixtext(GTK_CLIST(sess->chatwins[chatbox].clist), row, 0, listitem[0], 2, sess->pixmaps.pm_gold, sess->pixmaps.bm_gold );
	else gtk_clist_set_pixtext(GTK_CLIST(sess->chatwins[chatbox].clist), row, 0, listitem[0], 2, sess->pixmaps.pm_green, sess->pixmaps.bm_green );
}

void command_join_welcome(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	gchar *seplist[2];

	/* 0=Channel 1=Message */
	
	seperate_string(instr, (gchar **)seplist, 2);
	
	chatbox = get_chat_box_nr(sess, seplist[0]);
	
	if(chatbox != -1)
	{
		string_out(sess->chatwins[chatbox].ctext, seplist[1], 0xffff, 0, 0);
	}
	else
	{
		g_warning(WARNING_GET_CHANNEL, seplist[0]);
	}
}

void command_server_info(gtk_sess *sess, gchar *instr)
{
	gchar *msg;
	gchar *seplist[3];
	
	/* 0=Libraries 1=Songs 2=GB */

	msg = (gchar *)g_malloc(strlen(STATUSBAR_CONNECTED) + strlen(instr));
	seperate_string(instr, (gchar **)seplist, 3);	
	sprintf((gchar *)msg, STATUSBAR_CONNECTED, seplist[1], seplist[2], seplist[0]);
	gtk_statusbar_push(GTK_STATUSBAR(sess->statusbar), 1, msg);
	g_free(msg);
}

void command_fail(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	
	chatbox = get_chat_box_nr(sess, PRIVATE_CHAT_WINDOW);
	
	if(chatbox != -1)
	{
		string_out(sess->chatwins[chatbox].ctext, instr, 0, 0xffff, 0xffff);
	}
	else
	{
		g_warning(WARNING_GET_CHANNEL, (gchar *)PRIVATE_CHAT_WINDOW);
	}
}

void command_get_msg(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	gchar *seplist[3];
	gchar *outmsg;
	
	/* 0=Channel 1=User 2=Message */
	
	outmsg = (gchar *)g_malloc(strlen(instr) + 5);
	seperate_string(instr, (gchar **)seplist, 3);
	
	sprintf(outmsg, "<%s> %s", seplist[1], seplist[2]);

	chatbox = get_chat_box_nr(sess, seplist[0]);
	if(chatbox != -1)
	{
		string_out(sess->chatwins[chatbox].ctext, outmsg, 0, 0, 0);
	}
	else
	{
		g_warning(WARNING_GET_CHANNEL, seplist[0]);
	}
	g_free(outmsg);
}

void command_get_tell(gtk_sess *sess, gchar *instr)
{
	gint chatbox;
	gchar *outmsg;

	/* 0=User 1=Message */	

	gchar *seplist[2];

	outmsg = (gchar *)g_malloc(strlen(instr) + strlen(USER_TELL_MSG));
	seperate_string(instr, (gchar **)seplist, 2);
	
	sprintf(outmsg, USER_TELL_MSG, seplist[0], seplist[1]);
	
	chatbox = get_chat_box_nr(sess, PRIVATE_CHAT_WINDOW);
	if(chatbox != -1)
	{
		string_out(sess->chatwins[chatbox].ctext, outmsg, 0, 0xffff, 0);
	}
	else
	{
		g_warning(WARNING_GET_CHANNEL, (gchar *)PRIVATE_CHAT_WINDOW);
	}
	g_free(outmsg);
}

void command_verify_ul(gtk_sess *sess, gchar *instr)
{
	/* Has to be better! */
	send_string(sess, instr, 608);
}

void destroy_resume_list(gpointer data)
{
	gchar *temp = (gchar *)data;
	g_free(temp);
}

void command_resume_list(gtk_sess *sess, gchar *instr)
{
	gchar *seplist[7];
	gchar *inlist[4];
	gchar *tspeed;
	gchar *data;
	gint row;
	gint speed;
	
	if(!sess->resume.OPEN) return;
	seperate_string(instr, (gchar **)seplist, 7);

	/*if(strcasecmp((gchar *)sess->resume.md5, seplist[1]) != 0) return;*/
	
	data = (gchar *)g_malloc(strlen(seplist[3]) + 1);
	strcpy(data, seplist[3]);

	speed = atoi(seplist[6]);
	tspeed = get_speed(speed);

	inlist[0] = make_file(seplist[3]);
	inlist[1] = seplist[0];
	inlist[2] = tspeed;
	inlist[3] = seplist[5];
	
	row = gtk_clist_append(GTK_CLIST(sess->resume.clist), inlist);
	gtk_clist_set_row_data_full(GTK_CLIST(sess->resume.clist), row, data, destroy_resume_list);
}

void read_command_run(gtk_sess *sess, gint code, gchar *instr)
{
	switch(code)
	{
	case 201:
		command_search_results(sess, instr); break;
	
	case 202:	
		command_search_notfound(sess); break;
		
	case 204:
		get_file_ok(sess, instr); break;
	
	case 205:
		command_get_tell(sess, instr); break;
	
	case 209:
		command_hotlist_online(sess, instr); break;
		
	case 212:
		command_hotlist_results(sess, instr); break;
		
	case 213:
		command_hotlist_nofiles(sess, instr); break;
		
	case 214:
		command_server_info(sess, instr); break;
	
	case 216:
		command_resume_list(sess, instr); break;
	
	case 217:
		command_resume_notfound(sess); break;
	
	case 301:
		command_hl_useradd(sess, instr); break;
	
	case 403:
		command_get_msg(sess, instr); break;
		
	case 404:
		command_fail(sess, instr); break;

	case 406:
		command_connect(sess, instr); break;
		
	case 407:
		command_leave(sess, instr); break;
		
	case 408:
		command_join_list(sess, instr); break;
		
	case 405:
		make_chat_box(sess, instr, TRUE); break;
		
	case 410:
		command_join_welcome(sess, instr); break;
		
	case 409: break;
	
	case 501:
		firewall_connection(sess, instr); break;
	
	case 604:
		command_whois_ans(sess, instr); break;
		
	case 605:
		command_whois_offline_ans(sess, instr); break;

	case 607:
		command_verify_ul(sess, instr); break;

	case 8224: break;
		
	case 621:
		red_print(sess, instr);	break;
	
	case 751:
		ping_back(sess, instr, TRUE); break;
		
	case 752:
		ping_back(sess, instr, FALSE); break;

	default:
		unknown_command(sess, instr, code);
	}
}
