/* GTK Napster-0.301
 * Copyright (C) 2000 Unified (xilliator@hotmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>

#include "common.h"
#include "nconnect.h"

gint napster_findbestserver(gtk_sess *sess, gchar *ip, gboolean *CANCEL, time_t *start_time)
{
	struct sockaddr_in dest_addr;
	gchar buffer[21];
	gint length = 0;
	gint sockcon = 0;
   	gint i = 0;
	
	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(PORT);
	dest_addr.sin_addr.s_addr = inet_addr(IP);
	memset(dest_addr.sin_zero, 0, 8);
	
	if((sockcon = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
  	  	gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_SOCKET);
		return FALSE;
	}
	
	*start_time = time(NULL);

	switch(connect_with_select(sockcon, &dest_addr, *start_time, sess->timeout, CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_TIMEOUT);
			close(sockcon);
			return FALSE;
		case RECV_SELECT_ERROR:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_ERROR);
			close(sockcon);
			return FALSE;
		case RECV_SELECT_CANCEL:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_CANCEL);
			close(sockcon);
			return FALSE;
	}

	switch (length = recv_with_select(sockcon, (gchar *)buffer, 21, *start_time, sess->timeout, CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_TIMEOUT);
			close(sockcon);
			return FALSE;
		case RECV_SELECT_ERROR:
			close(sockcon);
			return FALSE;
		case RECV_SELECT_CANCEL:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_IP_CANCEL);
			close(sockcon);
			return FALSE;
	}

	close(sockcon);
	
	buffer[length] = '\0';

	while((buffer[i] != ':') && (i<length)) i++;
	if(i==length) return FALSE;
	
	buffer[i] = '\0';
	strcpy(ip, (gchar *)buffer);
	return atoi((gchar *)&buffer[i+1]);
}

gboolean napster_connect_to_server(gtk_sess *sess, gchar *ip, gint port, gboolean newuser, gboolean *CANCEL, time_t *start_time)
{
	struct sockaddr_in dest_addr;
	gchar *msg;
	gint16 length = 0;
	gint16 code = 0;
	
	#ifdef NAPSTER_DEBUG
	g_print("Connecting to IP: %s PORT %d\n", ip, port);
	#endif

	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(port);
	dest_addr.sin_addr.s_addr = inet_addr(ip);
	memset(dest_addr.sin_zero, 0, 8);
	
	if((sess->sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
  	  	gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_SOCKET);
		return FALSE;
	}
	
	switch(connect_with_select(sess->sockfd, &dest_addr, *start_time, sess->timeout, CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_CONNECT_TIMOUT);
			close(sess->sockfd);
			return FALSE;
		case RECV_SELECT_ERROR:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_CONNECT_ERROR);
			close(sess->sockfd);
			return FALSE;
		case RECV_SELECT_CANCEL:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_CONNECT_CANCEL);
			close(sess->sockfd);
			return FALSE;
	}

	if(newuser)
	{
		if(!send_string(sess, (gchar *)sess->username, 7))
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_USERNAME);
			close(sess->sockfd);
			return FALSE;
		}

		while(gtk_events_pending())
			gtk_main_iteration();

		msg = (gchar *)g_malloc(145 + strlen((gchar *)LOGIN_ERROR_USERINUSE) + strlen((gchar *)VERISON_INFO_SEND));

		switch (recv_with_select(sess->sockfd, (gchar *)msg, 6, *start_time, sess->timeout, CANCEL))
		{
			case RECV_SELECT_TIMOUT:
				gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_VER_TIMEOUT);
				close(sess->sockfd);
				g_free(msg);
				return FALSE;
			case RECV_SELECT_ERROR:
				close(sess->sockfd);
				g_free(msg);
				return FALSE;
			case RECV_SELECT_CANCEL:
				gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_VER_CANCEL);
				close(sess->sockfd);
				g_free(msg);
				return FALSE;
		}

		if(msg[2] == 9)
		{
			sprintf(msg, LOGIN_ERROR_USERINUSE, (gchar *)sess->username);
			gtk_label_set_text(GTK_LABEL(sess->login_label), msg);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		}
				
		while(gtk_events_pending())
			gtk_main_iteration();

		sprintf(msg, "%s %s %d \"%s\" %d %s", (gchar *)sess->username, (gchar *)sess->password, sess->FIREWALL?0:sess->port, (gchar *)VERISON_INFO_SEND, sess->speed, sess->email);
		
		if(!send_string(sess, msg, 6))
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_USERNAME_PASSWORD);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		}
	}
	else
	{
		msg = (gchar *)g_malloc(strlen((gchar *)sess->username) + strlen((gchar *)sess->password) + 22);
		sprintf(msg, "%s %s %d \"%s\" %d", (gchar *)sess->username, (gchar *)sess->password, sess->FIREWALL?0:sess->port, (gchar *)VERISON_INFO_SEND, sess->speed);

		if(!send_string(sess, msg, 2))
		{
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_ERROR_USERNAME_PASSWORD);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		}
 	}
 	
	gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_PASSWORD);

	switch (recv_with_select(sess->sockfd, (gchar *)msg, 4, *start_time, sess->timeout, CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_TIMEOUT);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		case RECV_SELECT_ERROR:
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		case RECV_SELECT_CANCEL:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_CANCEL);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
	}
	
	memcpy(&length, msg, 2);
	memcpy(&code, &msg[2], 2);
	msg = (gchar *)g_realloc(msg, length + 1);

	switch (recv_with_select(sess->sockfd, (gchar *)msg, length, *start_time, sess->timeout, CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_TIMEOUT);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		case RECV_SELECT_ERROR:
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
		case RECV_SELECT_CANCEL:
			gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_CANCEL);
			close(sess->sockfd);
			g_free(msg);
			return FALSE;
	}

	if(code != 3)
	{
		gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_USERNAME_WRONG);
		close(sess->sockfd);
		g_free(msg);
		return FALSE;
	}
	
	g_free(msg);
	hl_send_logon_request(NULL, (gpointer)sess);
	gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_SENDING_SHARES);
	make_share_list(sess);
	sess->sockid = gdk_input_add(sess->sockfd, GDK_INPUT_READ, GTK_SIGNAL_FUNC(napster_something_to_read), sess);
	return TRUE;
}

gboolean napster_connect(gtk_sess *sess, gboolean newuser, gboolean *LOCK, gboolean *CANCEL, gchar *ip, gint port)
{
	gchar bestip[16];
  	gint bestport = 0;
  	time_t start_time;
	
	sess->CONNECTED = TRUE;
	*LOCK = TRUE;

	if(port == 0)
	{	
		bestport = napster_findbestserver(sess, bestip, CANCEL, &start_time);
	}
	else
	{
		start_time = time(NULL);
		sprintf((gchar *)bestip, "%s", ip);
		bestport=port;
	}
	
  	if(bestport != FALSE) 
  	{
  		gtk_label_set_text(GTK_LABEL(sess->login_label), LOGIN_CONNECTING);

		while(gtk_events_pending())
			gtk_main_iteration();

		if(napster_connect_to_server(sess, bestip, bestport, newuser, CANCEL, &start_time))
  		{
			make_chat_box(sess, PRIVATE_CHAT_WINDOW, FALSE);
			*LOCK = FALSE;
			*CANCEL = FALSE;
			return TRUE;
		}
		else 
		{
			sess->CONNECTED = FALSE;
			*LOCK = FALSE;
			*CANCEL = FALSE;
			return FALSE;
		}
	}
	else 
	{
		sess->CONNECTED = FALSE;
		*LOCK = FALSE;
		*CANCEL = FALSE;
		return FALSE;
	}
}
