/* GTK Napster-0.301
 * Copyright (C) 2000 Unified (xilliator@hotmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <errno.h>

#include "common.h"
#include "nread.h"

gboolean napster_something_to_read(gpointer data, gint source, GdkInputCondition condition)
{
	gtk_sess *sess = (gtk_sess *)data;
		
	#ifdef NAPSTER_DEBUG
	g_print("In napster_something_to_read: Received=%d, toreceive=%d, code=%d\n", sess->received, sess->toreceive, sess->code);
	#endif
	
	if(sess->received < 4)
	{
		if((sess->received += recv(sess->sockfd, &sess->buffer[sess->received], 4-sess->received, 0)) == -1)
		{
			perror("recv");
			gtk_message_error(MSGBOX_ERROR_LENGTH_CODE, MSGBOX_ERROR_RECV_TITLE);
			return FALSE;
		}
		#ifdef NAPSTER_DEBUG
		g_print("Received is: %d\n", sess->received);
		#endif
		if(sess->received == 4)
		{
			memcpy(&sess->toreceive, sess->buffer, 2);
			memcpy(&sess->code, &sess->buffer[2], 2);
		}
	}

	if(sess->received >= 4)
	{
		#ifdef NAPSTER_DEBUG	
		g_print("Code is: %d\n", sess->code);
		g_print("Toreceive is: %d\n", sess->toreceive);
		#endif
		if((sess->received += recv(sess->sockfd, &sess->buffer[sess->received-4], (sess->toreceive)-(sess->received-4), 0)) == -1)
		{
			perror("recv");
			gtk_message_error(MSGBOX_ERROR_LENGTH_CODE, MSGBOX_ERROR_RECV_TITLE);
			return FALSE;
		}
		if((sess->received-4) == (sess->toreceive))
		{
			sess->buffer[sess->toreceive] = '\0';
			#ifdef NAPSTER_DEBUG	
			g_print("Recv: %d %s\n", sess->code, sess->buffer);
			#endif
			read_command_run(sess, sess->code, sess->buffer);
			sess->code = sess->toreceive = sess->received = 0;
			return TRUE;
		}
	}
	return FALSE;
}

gint send_string(gtk_sess *sess, gchar *instr, gint16 code)
{
	gchar *msg;
	gint16 length = strlen(instr);
	gint sendt = 0;
	
	msg = (gchar *)g_malloc(length + 6);

	memcpy(msg, &length, 2);
	memcpy(&msg[2], &code, 2);
	strcpy(&msg[4], instr);
	#ifdef NAPSTER_DEBUG	
	g_print("Sending: %s\n", &msg[4]);
	#endif
	if(sess->CONNECTED)
	{
		while(sendt < (length+4))
		{
			if((sendt += send(sess->sockfd, msg, length+4, 0)) == -1)
			{
				perror("send");
				g_free(msg);
				return FALSE;
			}
		}
	}
	else
	{
		gtk_message_warning(MSGBOX_NOT_CONNECTED, MSGBOX_ERROR_SEND_TITLE);
		g_free(msg);
		return FALSE;
	}
	g_free(msg);
	return TRUE;
}

gint recv_with_select(gint socketfd, gchar *buffer, gint sizetoread, time_t timeout, gint timeoutafter, gboolean *iCANCEL)
{
	fd_set readfds;
   	struct timeval tv;
	/*gboolean CANCEL;
  	memcpy(&CANCEL, iCANCEL, sizeof(gboolean));*/

	for(;;)
	{
		tv.tv_sec = 0;
		tv.tv_usec = 1000;
		
	  	FD_ZERO(&readfds);
	   FD_SET(socketfd, &readfds);
		
		if((select(socketfd+1, &readfds, NULL, NULL, &tv)) == -1)
		{
			perror("select");
			return RECV_SELECT_ERROR;
		}
		if(FD_ISSET(socketfd, &readfds)) 
		{		
			return recv(socketfd, buffer, sizetoread, 0);
		}
		else
		{
			while(gtk_events_pending())
				gtk_main_iteration();
			
			if(*iCANCEL)
			{
				return RECV_SELECT_CANCEL;
			}
			if(difftime(time(NULL), timeout) > timeoutafter)
			{
				return RECV_SELECT_TIMOUT;
			}
		}
	}
}

gint connect_with_select(gint socketfd, gpointer dest_addr, time_t timeout, gint timeoutafter, gboolean *iCANCEL)
{
	fd_set writefds;
    struct timeval tv;
    gint old, val, status, gintsize;
 	old = fcntl(socketfd, F_GETFL);
  	fcntl(socketfd, F_SETFL, O_NONBLOCK | old);
	
	if(connect(socketfd, (struct sockaddr *) dest_addr, sizeof(struct sockaddr)) == 0)
	{
		perror("select");
		return RECV_SELECT_ERROR;
	}

	if(errno == EALREADY || errno == EINPROGRESS)
	{
		while(1)
		{
			tv.tv_sec = 0;
			tv.tv_usec = 1000;
			
		  	FD_ZERO(&writefds);
		    FD_SET(socketfd, &writefds);
			
			if((val = select(socketfd+1, NULL, &writefds, NULL, &tv)) == -1)
			{
				perror("select");
				return RECV_SELECT_ERROR;
			}
			#ifdef NAPSTER_DEBUG
			g_print("Value: %d\n", val);
			#endif			
			if(val)
			{
				gintsize = sizeof(gint);
				getsockopt(socketfd, SOL_SOCKET, SO_ERROR, &status, &gintsize);
				if(!status)
				{
				  	fcntl(socketfd, F_SETFL, old);
				  	return TRUE;
				}
				else
				{
					g_warning(strerror(status));
					return RECV_SELECT_ERROR;
				}
			}
			
			while(gtk_events_pending())
				gtk_main_iteration();
				
			if(*iCANCEL)
			{
				return RECV_SELECT_CANCEL;
			}
			
			if(difftime(time(NULL), timeout) > timeoutafter)
			{
				return RECV_SELECT_TIMOUT;
			}
		}
	}
  	fcntl(socketfd, F_SETFL, old);
  	return 1;
}