/* GTK Napster-0.301
 * Copyright (C) 2000 Unified (xilliator@hotmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include <string.h>
#include <stdlib.h>
#include "common.h"
#include "sort.h"

void chat_sort_column(GtkCList *clist, gint column, gpointer user_data)
{
	gtk_clist_set_sort_column(clist, column);
	switch (column)
	{
		case 0: gtk_clist_set_compare_func(clist, text_compare); break;
		case 1: gtk_clist_set_compare_func(clist, number_compare); break;
		case 2: gtk_clist_set_compare_func(clist, line_speed_compare); break;
	}
	gtk_clist_sort(clist);
}

void search_sort_column(GtkCList *clist, gint column, gpointer user_data)
{
	gtk_clist_set_sort_column(clist, column);
	switch (column)
	{
		case 0: 
		case 5: gtk_clist_set_compare_func(clist, text_compare); break;
		case 2:
		case 3: gtk_clist_set_compare_func(clist, number_compare); break;
		case 6: gtk_clist_set_compare_func(clist, line_speed_compare); break;
		default: return;
	}
	gtk_clist_sort(clist);
}

void hotlist_sort_column(GtkCList *clist, gint column, gpointer user_data)
{
	gtk_clist_set_sort_column(clist, column);
	switch (column)
	{
		case 0: gtk_clist_set_compare_func(clist, text_compare); break;
		case 2:
		case 3: gtk_clist_set_compare_func(clist, number_compare); break;
		default: return;
	}
	gtk_clist_sort(clist);
}

void library_sort_column(GtkCList *clist, gint column, gpointer user_data)
{
	gtk_clist_set_sort_column(clist, column);
	switch (column)
	{
		case 0: gtk_clist_set_compare_func(clist, text_compare); break;
		case 2:
		case 3: gtk_clist_set_compare_func(clist, number_compare); break;
		default: return;
	}
	gtk_clist_sort(clist);
}

void transfer_sort_column(GtkCList *clist, gint column, gpointer user_data)
{
	gtk_clist_set_sort_column(clist, column);
	switch (column)
	{
		case 0: 
		case 3:
		case 5: gtk_clist_set_compare_func(clist, text_compare); break;
		case 4: gtk_clist_set_compare_func(clist, line_speed_compare); break;
		default: return;
	}
	gtk_clist_sort(clist);
}

void hluser_sort_column(GtkCList *clist, gint column, gpointer user_data)
{
	gtk_clist_set_sort_column(clist, column);
	gtk_clist_set_compare_func(clist, text_compare);
	gtk_clist_sort(clist);
}

gint text_compare(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	char *text1 = NULL; 
	char *text2 = NULL; 

	GtkCListRow *row1 = (GtkCListRow *) ptr1; 
	GtkCListRow *row2 = (GtkCListRow *) ptr2; 

	switch (row1->cell[clist->sort_column].type) 
	{ 
	case GTK_CELL_TEXT: 
		text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text; 
		break; 
	case GTK_CELL_PIXTEXT: 
		text1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text; 
		break; 
		default: 
		break; 
	} 
	switch (row2->cell[clist->sort_column].type) 
	{ 
	case GTK_CELL_TEXT: 
		text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text; 
		break; 
	case GTK_CELL_PIXTEXT: 
		text2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text; 
		break; 
		default: 
		break; 
	} 

	if(text1 == NULL) return -1;
	else if(text2 == NULL) return 0;
	else if(g_strcasecmp(text1, CLIST_SEARCH_FINISHED) == 0) return -1;
	else if(g_strcasecmp(text2, CLIST_SEARCH_FINISHED) == 0) return 1;
	else if(g_strcasecmp(text1, CLIST_SEARCH_SEARCHING) == 0) return -1;
	else if(g_strcasecmp(text2, CLIST_SEARCH_SEARCHING) == 0) return 1;
	else if(g_strcasecmp(text1, HL_CLIST_START) == 0) return -1;
	else if(g_strcasecmp(text2, HL_CLIST_START) == 0) return 1;
	else if(g_strcasecmp(text1, HL_CLIST_FINISHED) == 0) return -1;
	else if(g_strcasecmp(text2, HL_CLIST_FINISHED) == 0) return 1;

	return g_strcasecmp(text1, text2); 
}

gint line_speed_compare(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	gint iptr1 = 10;
	gint iptr2 = 10;
	char *text1 = NULL; 
	char *text2 = NULL; 

	GtkCListRow *row1 = (GtkCListRow *) ptr1; 
	GtkCListRow *row2 = (GtkCListRow *) ptr2; 
		
	text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text; 
	text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text; 

	if(text1 == 0) return -1;
	else if(text2 == 0) return 0;
	
	if(strcasecmp(LINE_SPEED_UNKNOWN, (gchar *)text1) == 0) iptr1 = 0;
	else if(strcasecmp(LINE_SPEED_14MODEM, (gchar *)text1) == 0) iptr1 = 1;
	else if(strcasecmp(LINE_SPEED_28MODEM, (gchar *)text1) == 0) iptr1 = 2;
	else if(strcasecmp(LINE_SPEED_33MODEM, (gchar *)text1) == 0) iptr1 = 3;
	else if(strcasecmp(LINE_SPEED_56MODEM, (gchar *)text1) == 0) iptr1 = 4;
	else if(strcasecmp(LINE_SPEED_ISDN56, (gchar *)text1) == 0) iptr1 = 5;
	else if(strcasecmp(LINE_SPEED_ISDN128, (gchar *)text1) == 0) iptr1 = 6;
	else if(strcasecmp(LINE_SPEED_CABLE, (gchar *)text1) == 0) iptr1 = 7;
	else if(strcasecmp(LINE_SPEED_DSL, (gchar *)text1) == 0) iptr1 = 8;
	else if(strcasecmp(LINE_SPEED_T1, (gchar *)text1) == 0) iptr1 = 9;
	else if(strcasecmp(LINE_SPEED_T3, (gchar *)text1) == 0) iptr1 = 10;
	
	if(strcasecmp(LINE_SPEED_UNKNOWN, (gchar *)text2) == 0) iptr2 = 0;
	else if(strcasecmp(LINE_SPEED_14MODEM, (gchar *)text2) == 0) iptr2 = 1;
	else if(strcasecmp(LINE_SPEED_28MODEM, (gchar *)text2) == 0) iptr2 = 2;
	else if(strcasecmp(LINE_SPEED_33MODEM, (gchar *)text2) == 0) iptr2 = 3;
	else if(strcasecmp(LINE_SPEED_56MODEM, (gchar *)text2) == 0) iptr2 = 4;
	else if(strcasecmp(LINE_SPEED_ISDN56, (gchar *)text2) == 0) iptr2 = 5;
	else if(strcasecmp(LINE_SPEED_ISDN128, (gchar *)text2) == 0) iptr2 = 6;
	else if(strcasecmp(LINE_SPEED_CABLE, (gchar *)text2) == 0) iptr2 = 7;
	else if(strcasecmp(LINE_SPEED_DSL, (gchar *)text2) == 0) iptr2 = 8;
	else if(strcasecmp(LINE_SPEED_T1, (gchar *)text2) == 0) iptr2 = 9;
	else if(strcasecmp(LINE_SPEED_T3, (gchar *)text2) == 0) iptr2 = 10;
			
	return (iptr2-iptr1);
}

gint number_compare(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
	gint lptr1 = 0;
	gint lptr2 = 0;
	gchar *text1 = NULL;
	gchar *text2 = NULL;
		
	GtkCListRow *row1 = (GtkCListRow *) ptr1; 
	GtkCListRow *row2 = (GtkCListRow *) ptr2; 

	text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
	text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
		
	if(text1 == 0) return -1;
	else if(text2 == 0) return 0;
		
	lptr1 = atoi(text1); 
	lptr2 = atoi(text2); 

	return (lptr2-lptr1);
}



