/* GTK Napster-0.301
 * Copyright (C) 2000 Unified (xilliator@hotmail.com).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
 
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>

#include "common.h"
#include "ulsess.h"
#include "global.h"
#include "md5.h"

guint8 get_layer(guint8 inbyte)
{
	inbyte = (inbyte << 5);
	inbyte = (inbyte >> 6);
	return inbyte-1;
}

gint get_bitrate(gboolean version, guint8 inbyte, guint8 layer)
{
	if(version) return bitrate_mpeg1[layer][(gint)(inbyte >> 4) - 1];
	return bitrate_mpeg2[layer][(gint)(inbyte >> 4) - 1];
}

gint get_freq(gboolean version, guint8 inbyte)
{
	inbyte = (inbyte << 4);
	return (freq_table[version][(gint)(inbyte >> 6)]);
}

gboolean get_version(guint8 inbyte)
{
	inbyte = (inbyte << 4);
	return (gboolean)(inbyte >> 7);
}

void file_to_write(gpointer data, gint source, GdkInputCondition condition)
{
	gtk_ul_sess *ulsess = (gtk_ul_sess *)data;
	gchar buffer[4096];
	gchar percent_done[20];
	gint row = 0;
	gint percent;
	
	row = gtk_clist_find_row_from_data(GTK_CLIST(ulsess->sess->ul_list), ulsess);
	
	if((ulsess->size - ulsess->current) < 4096)
	{
		fread(buffer, 1, (ulsess->size - ulsess->current), ulsess->stream);
		ulsess->current = ulsess->size;
		if(send(ulsess->socketfd, buffer, (ulsess->size - ulsess->current), 0) == -1)
		{
			perror("recv");
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
			cancel_ul_sess(ulsess);
		}
		else
		{
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 2, "100%");
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_FINISH);
			cancel_ul_sess(ulsess);
		}
		return;
	}

	fread(buffer, 1, 4096, ulsess->stream);
	ulsess->current += 4096;
	if(send(ulsess->socketfd, buffer, 4096, 0) == -1)
	{
		perror("recv");
		gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		cancel_ul_sess(ulsess);
		return;	
	}
		
	percent = (gint)((ulsess->current*100)/ulsess->size);
	sprintf(percent_done, "%.0f (%d%%)", (gdouble)ulsess->size, percent);
	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 2, percent_done);
}

glong is_file_in_library(gtk_sess *sess, gchar *filename)
{
	gchar *text;
	gint row = 0;
	shared_parm *parm;
	
	while((gtk_clist_get_text(GTK_CLIST(sess->library_clist), row, 0, &text) != 0) &&
		(strcasecmp(text, filename) != 0))
	{
		row++;
		text = NULL;
	}
	if(text == NULL) return -1;
	
	parm = gtk_clist_get_row_data(GTK_CLIST(sess->library_clist), row);
	return parm->size;
}

void cancel_ul_sess(gtk_ul_sess *ulsess)
{
	if(ulsess->CONNECTING) ulsess->CANCEL=TRUE;
	if(ulsess->ACTIVE)
	{
		gdk_input_remove(ulsess->socketid);
		close(ulsess->socketfd);
		fclose(ulsess->stream);
		ulsess->ACTIVE = FALSE;
	}
} 

void destroy_ul_item(gpointer data)
{
	gtk_ul_sess *ulsess = (gtk_ul_sess *)data;
	cancel_ul_sess(ulsess);
	if(ulsess->CONNECTING)
	{
		ulsess->CLEAR=TRUE;
	}
	else
	{
		g_free(ulsess);
	}
}

void firewall_connection(gtk_sess *sess, gchar *instr)
{
	struct sockaddr_in dest_addr;
	unsigned short int sport;
	gtk_ul_sess *ulsess;
	gchar size[10];
	gchar *tsize;
	gint length, row;
	gchar *msg;
	gchar buffer[20];
	gint status;
	time_t start_time;
   	gchar *seplist[6];
	gchar *listitem[6];
	gchar *filename;

	/*0=nick 1=ip 2=port 3=filename 4=md5 5=speed*/

	seperate_string(instr, (gchar **)seplist, 6);
	ulsess = (gtk_ul_sess*)g_malloc(sizeof(gtk_ul_sess));
	memset(ulsess, 0, sizeof(gtk_ul_sess));	
	ulsess->sess = sess;
	ulsess->CONNECTING = TRUE;
	filename = (gchar *)g_malloc(strlen(seplist[3]) + 1);
	strcpy(filename, seplist[3]);
	filename = make_back_compatibel(filename);

	
	if((ulsess->size = is_file_in_library(sess, filename)) == -1)
	{
		g_warning("User %s tried to get file %s.", seplist[0], seplist[3]);
		g_free(ulsess);
		g_free(filename);
		return;
	}
	
	memset(listitem, 0, sizeof(listitem));
	
	sprintf((gchar *)size, "%.0f", (gdouble)ulsess->size);
	tsize = make_text_size(ulsess->size);

	listitem[0] = make_file(seplist[3]);
	if(ulsess->sess->SHOW_BYTE)	 listitem[1] = tsize;
	else listitem[1] = (gchar *)size;
	listitem[2] = "0%";
	listitem[3] = seplist[0];
	listitem[4] = get_speed(atoi(seplist[5]));
	listitem[5] = UPLOAD_STATUS_WAIT;
			
	row = gtk_clist_append(GTK_CLIST(sess->ul_list), listitem);
	gtk_clist_set_row_data_full(GTK_CLIST(sess->ul_list), row, ulsess, destroy_ul_item);

	g_free(tsize);
	
	sport = atoi(seplist[2]);
	
	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(sport);
		
	dest_addr.sin_addr.s_addr = (unsigned long int)strtoul(seplist[1], NULL, 10);	
	
	#ifdef LOCAL_TEST
	dest_addr.sin_addr.s_addr = inet_addr("192.168.0.2");
	#endif

	memset(dest_addr.sin_zero, 0, 8);
	
	#ifdef NAPSTER_DEBUG
	g_print("\nIP: %s PORT: %s\n", seplist[1], seplist[2]);
	#endif
	
	if((ulsess->socketfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
  	  	gtk_statusbar_push(GTK_STATUSBAR(ulsess->sess->statusbar), 1, UPLOAD_ERROR_UPLOAD);
  	  	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		g_free(filename);
		return;
	}
	
	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_CONNECTING);
	
	start_time = time(NULL);

	status = connect_with_select(ulsess->socketfd, &dest_addr, start_time, sess->timeout, &ulsess->CANCEL);
	
	if(ulsess->CLEAR)
	{
		close(ulsess->socketfd);
		free(ulsess);
		g_free(filename);
		return;
	}
	
	row = gtk_clist_find_row_from_data(GTK_CLIST(sess->ul_list), ulsess);
	
	switch(status)
	{
		case RECV_SELECT_TIMOUT:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_TIMEOUT);
			close(ulsess->socketfd);
			g_free(filename);
			return;
		case RECV_SELECT_ERROR:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
			close(ulsess->socketfd);
			g_free(filename);
			return;
		case RECV_SELECT_CANCEL:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_CANCEL);
			close(ulsess->socketfd);
			ulsess->CONNECTING = FALSE;
			g_free(filename);
			return;
	}

	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_SEND);
   
   	length = recv_with_select(ulsess->socketfd, (gchar *)buffer, 1, start_time, ulsess->sess->timeout, &ulsess->CANCEL);

	if(ulsess->CLEAR)
	{
		close(ulsess->socketfd);
		free(ulsess);
		g_free(filename);
		return;
	}

	row = gtk_clist_find_row_from_data(GTK_CLIST(sess->ul_list), ulsess);
   
	switch (length)
	{
		case RECV_SELECT_TIMOUT:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_TIMEOUT);
			close(ulsess->socketfd);
			g_free(filename);
			return;
		case RECV_SELECT_ERROR:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
			close(ulsess->socketfd);
			g_free(filename);
			return;
		case RECV_SELECT_CANCEL:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_CANCEL);
			close(ulsess->socketfd);
			ulsess->CONNECTING = FALSE;
			g_free(filename);
			return;			
	}
	
	if(buffer[0] != '1')
	{
		gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		close(ulsess->socketfd);
	}
		
	if((send(ulsess->socketfd, "SEND", 4, 0)) == -1)
	{
		perror("send");
  	  	gtk_statusbar_push(GTK_STATUSBAR(ulsess->sess->statusbar), 1, UPLOAD_ERROR_SEND);
  	  	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		close(ulsess->socketfd);
		g_free(filename);
		return;
	}

	msg = (gchar *)g_malloc(strlen(seplist[3])+strlen(size)+25);
	sprintf(msg, "%s \"%s\" %s", sess->username, seplist[3], size);
	
	if((send(ulsess->socketfd, msg, strlen(msg), 0)) == -1)
	{
		perror("send");
  	  	gtk_statusbar_push(GTK_STATUSBAR(ulsess->sess->statusbar), 1, UPLOAD_ERROR_FILENAME);
  	  	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		close(ulsess->socketfd);
		g_free(filename);
		return;
	}
	
	g_free(msg);
	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_OK);
	
	length = recv_with_select(ulsess->socketfd, (gchar *)buffer, 20, start_time, ulsess->sess->timeout, &ulsess->CANCEL);
	
	if(ulsess->CLEAR)
	{
		close(ulsess->socketfd);
		free(ulsess);
		g_free(filename);
		return;
	}

	row = gtk_clist_find_row_from_data(GTK_CLIST(sess->ul_list), ulsess);
	
	switch (length)
	{
		case RECV_SELECT_TIMOUT:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_TIMEOUT);
			close(ulsess->socketfd);
			g_free(filename);
			return;
		case RECV_SELECT_ERROR:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
			close(ulsess->socketfd);
			g_free(filename);
			return;
		case RECV_SELECT_CANCEL:
			gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_CANCEL);
			close(ulsess->socketfd);
			ulsess->CONNECTING = FALSE;
			g_free(filename);
			return;			
	}
	
	buffer[length] = '\0';

	if(strcmp(buffer, "INVALID REQUEST") == 0) 
	{
		gtk_statusbar_push(GTK_STATUSBAR(ulsess->sess->statusbar), 1, UPLOAD_ERROR_INVALID_REQUEST);
		gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		close(ulsess->socketfd);
		g_free(filename);
		return;
	}
	
	if(strcmp(buffer, "FILE NOT SHARED") == 0) 
	{
		gtk_statusbar_push(GTK_STATUSBAR(ulsess->sess->statusbar), 1, UPLOAD_ERROR_NOT_SHARED);
		gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		close(ulsess->socketfd);
		g_free(filename);
		return;
	}
	
	if(strcmp(buffer, "FILE NOT FOUND") == 0) 
	{
		gtk_statusbar_push(GTK_STATUSBAR(ulsess->sess->statusbar), 1, UPLOAD_ERROR_NOT_FOUND);
		gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		close(ulsess->socketfd);
		g_free(filename);
		return;
	}
	
	if((ulsess->stream = fopen(filename, "rb")) == NULL)
	{
		gtk_statusbar_push(GTK_STATUSBAR(ulsess->sess->statusbar), 1, UPLOAD_ERROR_FILE);
		perror("fopen");
		g_warning("Could not open %s for reading.", seplist[3]);
		g_free(msg);
		gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_ERROR);
		close(ulsess->socketfd);
		g_free(filename);
		return;
	}
	
	fseek(ulsess->stream, atol(buffer), SEEK_SET);
	ulsess->current = atol(buffer);
	ulsess->socketid = gdk_input_add(ulsess->socketfd, GDK_INPUT_WRITE, GTK_SIGNAL_FUNC(file_to_write), ulsess);	
	
	ulsess->CONNECTING = FALSE;
	ulsess->ACTIVE = TRUE;
	gtk_clist_set_text(GTK_CLIST(ulsess->sess->ul_list), row, 5, UPLOAD_STATUS_UPLOAD);
	g_free(filename);
}

void incomming_connection(gpointer data, gint source, GdkInputCondition condition)
{
	gtk_sess *sess = (gtk_sess *)data;	
	time_t start_time;
	gint length;
	gchar buffer[5];
	gint socketfd;
	gboolean CANCEL = FALSE;
	
	#ifdef NAPSTER_DEBUG	
	g_print("Someone's one to you!\n");
	#endif

	start_time = time(NULL);
	
	socketfd = accept(sess->listenfd, NULL,NULL);
	
	if((send(socketfd, "1", 1, 0)) == -1)
	{
		perror("send");
		close(socketfd);
		return;
	}

	switch (length = recv_with_select(socketfd, (gchar *)buffer, 4, start_time, sess->timeout, &CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			g_warning("Upload/Firewall request timed out.");
			close(socketfd);
			return;
		case RECV_SELECT_ERROR:
			g_warning("Upload/Firewall error.");
			close(socketfd);
			return;
	}

	buffer[length] = '\0';

	if(strcasecmp((gchar *)buffer, "GET") == 0) ul_incomming(sess, socketfd, start_time);
	else if(strcasecmp((gchar *)buffer, "SEND") == 0) dl_incomming(sess, socketfd, start_time);
	else 
	{
		close(socketfd);
		g_warning("Upload/Firewall command unknown: %s.", (gchar *)buffer);
	}
}

void ul_incomming(gtk_sess *sess, gint socketfd, time_t start_time)
{
	gtk_ul_sess *ulsess;
	gchar buffer[500];
	gchar *tsize;
	gchar *seplist[3];
	gchar size[10];
	gchar *listitem[6];
	gint row, length;
   	gchar *filename;

	/*	0=Username  1=File  2=Offset 	*/
	
	ulsess = (gtk_ul_sess *)g_malloc(sizeof(gtk_ul_sess));
	memset(ulsess, 0, sizeof(gtk_ul_sess));
	ulsess->sess = sess;
	ulsess->socketfd = socketfd;

	switch (length = recv_with_select(socketfd, (gchar *)buffer, 500, start_time, sess->timeout, &ulsess->CANCEL))
	{
		case RECV_SELECT_TIMOUT:
			g_warning("Upload request timed out.");
			close(socketfd);
			return;
		case RECV_SELECT_ERROR:
			g_warning("Upload error.");
			close(socketfd);
			return;
	}
	
	seperate_string((gchar *)buffer, (gchar **)seplist, 3);
	filename = (gchar *)g_malloc(strlen(seplist[1]) + 1);
	strcpy(filename, seplist[1]);
	filename = make_back_compatibel(filename);
	
	if((ulsess->size = is_file_in_library(sess, filename)) == -1)
	{
		g_warning("User %s tried to download %s", seplist[0], filename);	
		send(socketfd, "FILE NOT FOUND", 14, 0);
		close(socketfd);
		g_free(ulsess);
		g_free(filename);
		return;
	}
	
	sprintf((gchar *)size, "%.0f", (gdouble)ulsess->size);

	if((send(socketfd, size, strlen(size), 0)) == -1)
	{
		perror("send");
		close(socketfd);
		g_free(ulsess);
		g_free(filename);
		return;
	}
	
	if((ulsess->stream = fopen(filename, "rb")) == NULL)
	{
		g_warning("Can't open file: %s", filename);
		perror("fopen");
		close(socketfd);
		g_free(ulsess);
		g_free(filename);
		return;
	}

	fseek(ulsess->stream, atol(seplist[2]), SEEK_SET);
	ulsess->current = atol(seplist[2]);

	tsize = make_text_size(ulsess->size);

	listitem[0] = make_file(seplist[1]);
	if(sess->SHOW_BYTE) listitem[1] = tsize;
	else listitem[1] = (gchar *)size;
	listitem[2] = "0%";
	listitem[3] = seplist[0];
	listitem[4] = get_speed(0); /*Temporary...*/
	listitem[5] = "Uploading...";

	row = gtk_clist_append(GTK_CLIST(sess->ul_list), listitem);
	gtk_clist_set_row_data_full(GTK_CLIST(sess->ul_list), row, ulsess, destroy_ul_item);
	
	ulsess->ACTIVE = TRUE;
	ulsess->socketid = gdk_input_add(ulsess->socketfd, GDK_INPUT_WRITE, GTK_SIGNAL_FUNC(file_to_write), ulsess);
	g_free(filename);
}

void make_socket_listener(gtk_sess *sess)
{
	struct sockaddr_in my_addr;
	gint old;
	
	my_addr.sin_family = AF_INET;
	my_addr.sin_port = htons(sess->port);
	my_addr.sin_addr.s_addr = INADDR_ANY;
	memset(my_addr.sin_zero, 0, 8);

	if((sess->listenfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		perror("socket");
  	  	gtk_label_set_text(GTK_LABEL(sess->login_label), "Error listening for connections!");
		return;
	}
	
	old = fcntl(sess->listenfd, F_GETFL);
  	fcntl(sess->listenfd, F_SETFL, O_NONBLOCK | old);
  	
  	if (bind(sess->listenfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) == -1)
	{
   	perror("bind");
   	close(sess->listenfd);
   	return;
   }

   listen(sess->listenfd, 10);
   
   sess->listenid = gdk_input_add(sess->listenfd, GDK_INPUT_READ, GTK_SIGNAL_FUNC(incomming_connection), sess);
}

gchar *make_md5(FILE *stream, glong size)
{
	MD5_CTX context;
  	guint8 digest[16];
  	gchar result[45];
  	guint8 buffer[8];
  	gint i, nritems;
  	glong byte;

	MD5Init (&context);
	
	for(byte = 0; byte < 300032; byte+=8)
	{
		nritems = fread((guint8 *)buffer, 1, 8, stream);
		MD5Update (&context, buffer, nritems);
		if(nritems < 8) break;
	}
  	
	MD5Final (digest, &context);
	
	for(i=0; i<16; i++)
		sprintf((gchar *)&result[i*2], "%02x", (guint8)digest[i]);
		
	sprintf((gchar *)result+32, "-%.0f", (gdouble)size);
	
	return g_strdup((gchar *)result);
}

void destroy_library_item(gpointer data)
{
	shared_parm *parm = (shared_parm *)data;
	g_free(parm->md5);
	g_free(parm);
}

void destroy_shared_item(gpointer data)
{
	g_free((glong *)data);
}

gboolean file_there(gchar *filename)
{
	if(fopen(filename, "r") == NULL) return FALSE;
	return TRUE;
}

gchar *make_compatibel(gchar *filename)
{
	gint i=0;
		
	while(filename[i] != '\0')
	{
		if(filename[i] == '/') filename[i] = '\\';
		i++;
	}
	return filename;
}

gchar *make_back_compatibel(gchar *filename)
{
	gint i=0;
		
	while(filename[i] != '\0')
	{
		if(filename[i] == '\\') filename[i] = '/';
		i++;
	}
	return filename;
}

void make_share_list(gtk_sess* sess)
{
	gchar buffer[1024];
	FILE *sharefile;
	gchar *seplist[6];
	gchar *inlist[5];
	gint row;
	gchar *time;
	gchar *tsize;
	shared_parm *parm;
	gchar *sendmsg;
		
	if((sharefile = fopen((gchar *)sess->sharedir, "r")) == NULL)
	{
		g_warning(WARNING_FILE_NOT_FOUND, (gchar *)sess->sharedir);
		return;
	}
		
	while(fgets(buffer, 1024, sharefile) != NULL)
	{
		buffer[strlen(buffer)-1] = '\0';
		seperate_string(buffer, (gchar **)seplist, 6);
			
		if(file_there(seplist[0]))
		{
			parm = (shared_parm *)g_malloc(sizeof(shared_parm));
			parm->md5 = (gchar *)g_malloc(strlen(seplist[1])+1);
			strcpy(parm->md5, seplist[1]);
			parm->size = atol(seplist[2]);
			parm->time = atoi(seplist[5]);
				
			time = make_text_time((glong)atoi(seplist[5]), sess->SHOW_DHMS);
			tsize = make_text_size((glong)atoi(seplist[2]));
	
			inlist[0] = seplist[0];
			inlist[1] = tsize;
			inlist[2] = seplist[3];
			inlist[3] = seplist[4];
			inlist[4] = time;
			
			row = gtk_clist_append(GTK_CLIST(sess->library_clist), inlist);
			gtk_clist_set_row_data_full(GTK_CLIST(sess->library_clist), row, parm, destroy_shared_item);
				
			sendmsg = (gchar *)g_malloc(strlen(seplist[0]) + strlen(parm->md5) + 50);
			sprintf(sendmsg, "\"%s\" %s %s %s %s %s", make_compatibel(seplist[0]), seplist[1], seplist[2], seplist[3], seplist[4], seplist[5]);
			send_string(sess, sendmsg, 100);

			g_free(time);
			g_free(tsize);
			g_free(sendmsg);
		}
		else
		{
			g_warning("Can't find file: %s\n", seplist[0]);
		}
	}
	fclose(sharefile);
	make_socket_listener(sess);
}

void lib_remove_file(gtk_sess *sess, gint row)
{
	gchar *infile;
	gchar *filename;
		
	gtk_clist_get_text(GTK_CLIST(sess->library_clist), row, 0, &infile);
	filename = g_malloc(strlen(infile)+1);
	strcpy(filename, infile);
	filename = make_compatibel(filename);

	if(send_string(sess, filename, 102)) gtk_clist_remove(GTK_CLIST(sess->library_clist), row);
	g_free(filename);
}

void file_selection_changed(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
	filedlg_parm *parm = (filedlg_parm *)data;
	parm->selected_files = g_list_append(parm->selected_files, GINT_TO_POINTER(row));
}

void file_selection_unselected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data)
{
	filedlg_parm *parm = (filedlg_parm *)user_data;
	parm->selected_files = g_list_remove(parm->selected_files, GINT_TO_POINTER(row));
}

void lib_add_file(filedlg_parm *parm, gint row)
{
	FILE *stream;
	gboolean version;
	gint bitrate;
	gint freq, localrow;
	glong size, seconds;
	glong startpos;
	guint8 readbyte, last;
	gchar *inlist[5];
	gchar *tsize;
	gchar *time;
	gchar *md5;
	gchar tbitrate[4];
	gchar tfreq[7];
	shared_parm *sparm;
	gchar *sendmsg;
	gchar *infile;
	gchar *filename;
	gint8 layer=0;
		
	gtk_clist_get_text(GTK_CLIST(GTK_FILE_SELECTION(parm->file_selector)->file_list), row, 0, &infile);
	filename = g_malloc(strlen(parm->path) + strlen(infile)+1);
	
	sprintf(filename, "%s%s", parm->path, infile);

	g_print("%s\n", &filename[strlen(filename)-3]);
	g_print("%s\n", filename);

	if(((filename[strlen(filename)-3] != 'm') && (filename[strlen(filename)-3] != 'M')) ||
		((filename[strlen(filename)-2] != 'p') && (filename[strlen(filename)-2] != 'P')))
	{
		gtk_message_warning("Some files/The file you selected is not a mp3 file.", "Not a mp3 file!");
		return;
	}

	if((stream = fopen(filename, "rb")) != NULL)
	{
		last=0;
		while(!feof(stream))
		{
			fread((guint8 *)&readbyte, 1, 1, stream);
			if((last == 0xFF) && (readbyte > 0xF0) && (readbyte != 0xFF))
			{
				if((layer = get_layer(readbyte)) != -1) break;
			}
			last = readbyte;
		}
		if(!feof(stream))
		{
			startpos = ftell(stream)-2;
			
			version = get_version(readbyte);
			fread((guint8 *)&readbyte, 1, 1, stream);
			bitrate = get_bitrate(version, readbyte, layer);
			freq = get_freq(version, readbyte);
			fseek(stream, 0, SEEK_END);
			size = ftell(stream);
			fseek(stream, startpos, SEEK_SET);
			md5 = make_md5(stream, size);
				
			seconds = (((size*8)/1000)/bitrate);
			if(bitrate) time = make_text_time(seconds, parm->sess->SHOW_DHMS);
			else time = NULL;
			tsize = make_text_size(size);
				
			sprintf((gchar *)tbitrate,"%d", bitrate);
			sprintf((gchar *)tfreq,"%d", freq);
				
			inlist[0] = filename;
			inlist[1] = tsize;
			inlist[2] = (gchar *)tbitrate;
			inlist[3] = (gchar *)tfreq;
			inlist[4] = time;
			
			if(parm->sess->CONNECTED)
			{
				sparm = (shared_parm *)g_malloc(sizeof(shared_parm));
				sparm->md5 = md5;
				sparm->size = (gdouble)size;
				sparm->time = (gint)seconds;
				localrow = gtk_clist_append(GTK_CLIST(parm->sess->library_clist), inlist);
				gtk_clist_set_row_data_full(GTK_CLIST(parm->sess->library_clist), localrow, sparm, destroy_library_item);
			}
			else g_free(md5);
					
			sendmsg = (gchar *)g_malloc(strlen(filename) + strlen(md5) + 50);
			sprintf(sendmsg, "\"%s\" %s %.0f %d %d %.0f", make_compatibel(filename), md5, (gdouble)size, bitrate, freq, (gdouble)seconds);
			send_string(parm->sess, sendmsg, 100);
			g_free(sendmsg);
			g_free(time);
			g_free(tsize);
		}
		else g_warning("Reading file: %s", filename);
		fclose(stream);
	}
	else g_warning("Opening file: %s", filename);
	g_free(filename);
}

void filedlg_ok(GtkWidget *widget, gpointer data)
{
	filedlg_parm *parm = (filedlg_parm *)data;
	gint i;
		
	parm->path = gtk_file_selection_get_filename(GTK_FILE_SELECTION(parm->file_selector));
	for(i=strlen(parm->path); i>0; i--)
	{
		if(parm->path[i] == '/')
		{
			parm->path[i+1] = '\0';
			break;
		}
	}
				
	for(i=0; i<g_list_length(parm->selected_files); i++)
		lib_add_file(parm, GPOINTER_TO_INT(g_list_nth_data(parm->selected_files, i)));
		
	gtk_widget_destroy(parm->file_selector);
	g_free(parm);
}

void filedlg_cancel(GtkWidget *widget, gpointer data)
{
	filedlg_parm *parm = (filedlg_parm *)data;
	gtk_widget_destroy(parm->file_selector);
	g_free(parm);	
}


void lib_add(GtkWidget *widget, gpointer data)
{
	gtk_sess *sess = (gtk_sess *)data;
	filedlg_parm *parm;
	parm = (filedlg_parm *)g_malloc(sizeof(filedlg_parm));
	memset(parm, 0 , sizeof(filedlg_parm));
		
	parm->sess = sess;
   	parm->file_selector = gtk_file_selection_new("Choose files to share.");
	gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(parm->file_selector)->file_list), GTK_SELECTION_EXTENDED);
     
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(parm->file_selector)->ok_button), "clicked", GTK_SIGNAL_FUNC (filedlg_ok), parm);
    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(parm->file_selector)->cancel_button), "clicked", GTK_SIGNAL_FUNC (filedlg_cancel), parm);

	gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(parm->file_selector)->file_list), "select_row", GTK_SIGNAL_FUNC(file_selection_changed), parm);
	gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(parm->file_selector)->file_list), "unselect_row", GTK_SIGNAL_FUNC(file_selection_unselected), parm);
		
	gtk_widget_show(parm->file_selector);
}
